
#include "..\rNES.h"
#include <string.h>

// 3 tiles:
// tile 0 is blank
// tile 1 is the note top
// tile 2 is the note bottom
byte note[] = {
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, // one blank tile
	#include "note.tile.h"
};

// 64 tiles|characters
byte font[] = {
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, // one blank tile
	#include "font.h"
};

// char code to tile index conversion table 
byte charset[] = {
	#include "charset.h"
};


byte bkg_pal[] = { 0x0E, 0x20, 0x00, 0x00 };

byte spr_pal[] = { 0x0E, 0x19, 0x0B, 0x00 };

byte ticks2beat; // How many vblanks to wait before making a beat
byte ticks; // ticks (vblanks) counter
byte tempo; // beats per minute. Usually between [40..208]
byte beats; // beats in the bar [1..9]
byte beat; // current beat. [1..beats]
byte volume; // [0..15]
byte lastjoy, joy; // joypad states

byte dirty;
#define dirty_tempo		1
#define dirty_beats		2
#define dirty_volume	3

byte str_tempo[4];
byte str_beats[4];
byte str_volume[4];

void update_numbers()
{
	sprintf( str_tempo,  "%3d", tempo );	
	sprintf( str_beats,  "%3d", beats );
	sprintf( str_volume, "%3d", volume );
}
			
void main()
{
	tempo = 160;
	ticks = 0;
	beats = 4;
	beat  = 1;
	volume = 8;
	lastjoy = 0;
	joy = 0;
	dirty = 0;
	
	rNES_init();
	rNES_sound_init( rNES_snd_chn_1 );
	rNES_disable_graphics();
	
	rNES_copy_vram( bkg_pal, 4, rNES_back_palette );
	rNES_copy_vram( spr_pal, 4, rNES_sprite_palette );
	
	rNES_copy_tiles( font, 0, rNES_back_tiles, 64 );
	rNES_copy_tiles( note, 0, rNES_sprite_tiles, 3 );
	
	rNES_print( "NESTRONOME V 0.1", 8,  2, rNES_name_table_0, charset );
	rNES_print( "$,# TEMPO  ;", 8, 20, rNES_name_table_0, charset );
	rNES_print( "&,% BEATS  ;", 8, 22, rNES_name_table_0, charset );
	rNES_print( "B,A VOLUME ;", 8, 24, rNES_name_table_0, charset );
	rNES_print( "WWW.PETRUZA.COM.AR/RNES", 4, 27, rNES_name_table_0, charset );
	
	update_numbers();
	
	rNES_print( str_tempo, 21, 20, rNES_name_table_0, charset );
	rNES_print( str_beats, 21, 22, rNES_name_table_0, charset );
	rNES_print( str_volume, 21, 24, rNES_name_table_0, charset );

	rNES_enable_graphics();
	
	ticks2beat = 3600 / tempo;
		
	while(1)
	{
		joy = rNES_get_joy( rNES_JOYPAD_1 );
		
		if( rNES_A_Down(joy) && ! rNES_A_Down(lastjoy) && volume != 15 )
		{
			++volume;
			dirty = dirty_volume;
		}		
		else if( rNES_B_Down(joy) && ! rNES_B_Down(lastjoy) && volume != 0 )
		{
			--volume;
			dirty = dirty_volume;
		}
		
		else if( rNES_LEFT_Down(joy) && ! rNES_LEFT_Down(lastjoy) && beats != 1 )
		{
			--beats;
			dirty = dirty_beats;
		}
		else if( rNES_RIGHT_Down(joy) && ! rNES_RIGHT_Down(lastjoy) && beats != 9 )
		{
			++beats;
			dirty = dirty_beats;
		}
		
		if( rNES_DOWN_Down( joy ) && tempo != 40 )
		{
			--tempo;
			dirty = dirty_tempo;
			ticks2beat = 3600 / tempo;
		}
		else if( rNES_UP_Down( joy ) && tempo != 255 )
		{
			++tempo;
			dirty = dirty_tempo;
			ticks2beat = 3600 / tempo;
		}
		
		lastjoy = joy;
	
		++ticks;
		if( ticks >= ticks2beat )
		{
			ticks = 0;
			
			if( beat == 1 )
				rNES_sound( rNES_snd_chn_1, rNES_sndlen_6, volume, 1200 );
			else
				rNES_sound( rNES_snd_chn_1, rNES_sndlen_3, volume, 1020 );
			
			++beat;
			if( beat > beats )
				beat = 1;
		}
		
		if( dirty )
			update_numbers();
			
		rNES_waitvblank(); // ------------------- rNES_waitvblank ------------------- //
		
		switch( dirty )
		{
			case dirty_tempo:			
				rNES_print( str_tempo, 21, 20, rNES_name_table_0, charset );
				break;
				
			case dirty_volume:
				rNES_print( str_volume, 21, 24, rNES_name_table_0, charset );				
				break;
				
			case dirty_beats:
				rNES_print( str_beats, 21, 22, rNES_name_table_0, charset );
		}
		
		//render cleanup
		addr(0x2000) = bin(1,0,0,0,1,0,0,0);
		addr(0x2005) = 0;
		addr(0x2005) = 0;
		
		dirty = 0;
	}
}















