
/*********************************************************************
rNES - a NES library for cc65 - version 0.1

(c) 2009 Ernesto Borio A.K.A. Petruza
mail:	petruzanautico@yahoo.com.ar
web:	petruza.com.ar/rnes

This library was made while learning to program the NES, therefore
it's incomplete and not very efficient. Hopefully it will be more
accurate and efficient in future versions, including a total re-
write in assembler, some day. (while retaining the capability of
being linked to C code)

Use this software in any way you like, modify it, distribute it
and use it on your free or commercial projects. Keeping this header 
and mentioning the author would be great though.

Special thanks to the guys at cc65 mailing list and nesdev forum

*********************************************************************/

#include "rNES.h"

//---------------------------------------------------------------- rNES_get_joy

byte rNES_get_joy( word joypad )
{	
	// joypad is really the joypad memory address, namely 0x4016 for joypad 1 
	// or 0x4017 for joypad 2
	
	byte i = 9, joy_state = 0;
	
	// request to read the joypad's state, writing the magic word to 0x401* 
	// ( magic WORD, got it? )
	addr(joypad) = 0x01;
	addr(joypad) = 0x00;

	// each read to 0x401* yields the state of one button, so pack them as bits 
	// into joy_state
	while( --i != 0 ) // countdown 8 times
		joy_state = ( joy_state << 1 ) | ( addr( joypad ) & 1 );
		
	return joy_state;
}

//------------------------------------------------------------- rNES_sound_init

void rNES_sound_init( byte channels )
{
	if( channels & rNES_snd_chn_1 )
	{
		addr(0x4000) = 0;
		addr(0x4001) = 0;
		addr(0x4002) = 0;
		addr(0x4003) = 0;
	}
	if( channels & rNES_snd_chn_2 )
	{
		addr(0x4004) = 0;
		addr(0x4005) = 0;
		addr(0x4006) = 0;
		addr(0x4007) = 0;
	}
	addr(0x4015) = channels; // 'channels' has bits set for every channel to init
}

//------------------------------------------------------------------ rNES_sound

void rNES_sound( byte channel, byte length, byte volume, word wavelength )
{
	// only suitable for rectangular wave channels 1 & 2

	byte offset = ( channel - 1 ) << 2;
	addr( 0x4000 + offset ) = bin5(1,0,0,0,0) | ( 0xF & volume );
	addr( 0x4001 + offset ) = 0x7F; // blargg's sugestion
	addr( 0x4002 + offset ) = (byte) wavelength;
	addr( 0x4003 + offset ) = length | ( wavelength >>8 );
}

//------------------------------------------------------------- rNES_copy_tiles

void rNES_copy_tiles( byte* tiles, word start_from, word start_to, word quantity )
{
	byte *top; // first address not to be written
	
	// Each tile occupies 16 bytes. Hence the <<4 shifting ( multiply by 16 )
	
	// convert quantity from tile units to bytes.
	quantity <<= 4;
	
	// add to the base address, the relative address of starting tile
	tiles += start_from << 4;
	
	// add the quantity of bytes to write, to the starting address, to get the top
	top = tiles;
	top += (byte*) quantity;
	
	// specify video RAM address to write to, in program RAM 0x2006:
	
		// convert start_to from tile units to bytes (<<4) and extract high byte (>>8)
		addr(0x2006) = (byte)( start_to >>4 ); // ... the result is to shift only >>4
	
		// convert start_to from tile units to bytes again, this time only low byte
		addr(0x2006) = (byte)( start_to <<4 );
	
	// increment pointer from starting address to top address
	while( tiles != top )
	{
		// write to video RAM
		addr(0x2007) = *tiles; // write the byte pointed at by 'tiles'
		++tiles;
	}
}

//----------------------------------------------------------- rNES_copy_palette
/*
void rNES_copy_palette( byte *palette, byte start_from, byte start_to, byte quantity )
{
	byte *top; // first address not to be written

	// add to the base address, the relative address of starting colour
	palette += start_from;
	
	// add the quantity of bytes to write, to the starting address, to get the top
	top = palette;
	top += (byte*) quantity;
	
	// specify video RAM address to write to, in program RAM 0x2006:
	
		// starting address of palettes is video RAM 0x3F00
		addr(0x2006) = 0x3F;
		
		// destination start address
		addr(0x2006) = start_to;
	
	// increment pointer from starting address to top address	
	while( palette != top )
	{
		addr(0x2007) = *palette; // write the byte pointed at by 'palette'
		++palette;
	}
}
*/

//-------------------------------------------------------------- rNES_fill_vram

void rNES_fill_vram( word address, word length, byte fill )
{
	length += address;
	addr(0x2006) = highbyte( address );
	addr(0x2006) = lowbyte( address );
	while( address != length )
	{
		addr(0x2007) = fill;
		++address;
	}
}

//------------------------------------------------------- rNES_disable_graphics

void rNES_disable_graphics()
{
	addr(0x2000) = 0;
	addr(0x2001) = 0;
}
	
//-------------------------------------------------------- rNES_enable_graphics

void rNES_enable_graphics()
{
	addr(0x2000) = bin(1,0,0,0,1,0,0,0);
	addr(0x2001) = bin(0,0,0,1,1,1,1,0);
}

//------------------------------------------------------------------- rNES_init

void rNES_init()
{
	rNES_waitvblank();
	rNES_waitvblank();
}

//------------------------------------------------------------- rNES_sprite_dma

void rNES_sprite_dma( byte page, byte start )
{
	addr(0x2003) = start;
	addr(0x4014) = page;
}

//--------------------------------------------------------------- rNES_fill_ram

void rNES_fill_ram( word address, word length, byte fill )
{
	length += address;
	while( address != length )
	{
		addr(address) = fill;
		++address;
	}
}

//-------------------------------------------------------------- rNES_copy_vram

void rNES_copy_vram( byte *source, word length, word target )
{
	byte *top = source;
	top += (byte*)length;
	addr(0x2006) = highbyte( (word)target );
	addr(0x2006) = lowbyte( (word)target );
	while( source != top )
	{
		addr(0x2007) = *source;
		++source;
	}
}

//------------------------------------------------------------------ rNES_print

void rNES_print( byte* string, byte x, byte y, word name_table_addr, byte* charset )
{
	name_table_addr += ( x + y * 32 );	
	addr(0x2006) = highbyte( name_table_addr );
	addr(0x2006) = lowbyte( name_table_addr );
	
	while( *string ) // while no null char ('\0') is found in the string
	{
		addr(0x2007) = charset[ *string - 32 ]; // translate ASCII so space (32) is zero
		++string;
	}
}









